from __future__ import print_function
import os
import sys
import argparse
import json
import urllib2
import time
import yaml
import acmagent
from acmagent import request
from acmagent import confirm


logger = acmagent.configure_logger('acmagent')

class ParseJsonInput(argparse.Action):
    """
    Parse json input file for the request-certificate command
    """
    def __init__(self, option_strings, dest, nargs=None, **kwargs):
        if nargs is not None:
            raise ValueError("nargs not allowed")
        super(ParseJsonInput, self).__init__(option_strings, dest, **kwargs)

    def __call__(self, parser, namespace, value, option_string=None):
        try:
            logger.debug('Opening input json file'.format(value))
            certificate_args = json.loads(urllib2.urlopen(value).read())
            setattr(namespace, self.dest, certificate_args)
        except urllib2.URLError:
            logger.exception('Failed reading json input')
            parser.error('Specified file "{}" is not readable'.format(value))
        except ValueError:
            if not value.startswith('file:'):
                logger.exception('Input json file is missing file scheme')
                parser.error('Specified file "{}" is missing file URL scheme'.format(value))
            else:
                logger.exception('Input json file is not valid json')
                parser.error('Specified file "{}" is not valid json'.format(value))


class ParseIMAPCredentials(argparse.Action):
    """
    Parse YAML IMAP credentials file for the confirm-certificate command
    """
    def __init__(self, option_strings, dest, nargs=None, **kwargs):
        if nargs is not None:
            raise ValueError("nargs not allowed")
        super(ParseIMAPCredentials, self).__init__(option_strings, dest, **kwargs)

    def __call__(self, parser, namespace, value, option_string=None):
        try:
            logger.debug('Opening IMAP credentials file'.format(value))
            imap_credentials = yaml.load(urllib2.urlopen(value).read())
            setattr(namespace, self.dest, imap_credentials)
        except urllib2.URLError as e:
            logger.exception('Failed reading json input')
            parser.error('Specified file "{}" is not readable'.format(value))
        except ValueError as e:
            if not value.startswith('file:'):
                logger.exception('Input json file is missing file scheme')
                parser.error('Specified file "{}" is missing file URL scheme'.format(value))
            else:
                logger.exception('Input json file is not valid YAML')
                parser.error('Specified file "{}" is not valid YAML'.format(value))


def _confirm_cert(args, parser):
    """
    Confirm ACM issued certificate

    :param args: cli arguments
    :return: None
    """
    try:
        imap_credentials = args.credentials if args.credentials else acmagent.load_imap_credentials()
        with confirm.ConfirmCertificate(imap_credentials) as acm_certificate_confirm:
            attempts_left = args.attempts
            while attempts_left:
                logger.debug('Starting ACM request for {} certificate, attempts left: {}, pause: {} seconds'.format(
                    args.certificate_id, attempts_left, args.wait))
                attempts_left -= 1
                time.sleep(args.wait)
                try:
                    success = acm_certificate_confirm.confirm_certificate(args.certificate_id)
                    if success:
                        parser.exit(0, 'Success: certificate has been confirmed\n')
                except acmagent.NoEmailsFoundException as e:
                    if attempts_left:
                        continue
                    parser.error(str(e))
                except acmagent.ACManagerException as e:
                    parser.error(str(e))
    except acmagent.ACManagerException as e:
        parser.error(str(e))


def _request_cert(args, parser):
    """
    Send a request to the ACM to issue SSL certificate

    :param args: cli arguments
    :return: None
    """
    if args.generate_cli_skeleton:
        json_file = request.Certificate.template()
        logger.debug('Generating json input file: {}'.format(json_file))
        parser.exit(0, json_file + "\n")

    if args.cli_input_json:
        try:
            certificate = request.Certificate.from_json_input(args.cli_input_json)
            acm_certificate = dict(certificate)
        except acmagent.InvalidCertificateJsonFileException as e:
            parser.error(str(e))
    else:
        if not args.domain_name:
            parser.error('--domain-name is required')

        certificate = request.Certificate(args.__dict__)
        acm_certificate = dict(certificate)

    acm_certificate_request = request.RequestCertificate()

    try:
        logger.debug('Requesting certificate: {}'.format(json.dumps(acm_certificate)))
        response = acm_certificate_request.request_certificate(acm_certificate)
    except Exception as e:
        logger.exception('Boto3 exception')
        parser.error(str(e))

    certificate_id = (response['CertificateArn'].split('/')[-1])
    logger.debug('Success, {} certificate was issued, id: {}'.format(acm_certificate['DomainName'], certificate_id))
    parser.exit(0, certificate_id)


def _setup_argparser():
    """
    Argparse factory

    :return: argparse.ArgumentParser
    """
    parser = argparse.ArgumentParser(description='ACM agent')
    subparsers = parser.add_subparsers(
        title='ACM agent - automates ACM certificates',
        description='ACM agents provides functionality to request and confirm ACM certificates using the CLI interface')

    request_cert_parser = subparsers.add_parser('request-certificate')
    request_cert_parser.set_defaults(func=_request_cert)
    request_cert_parser.add_argument('--domain-name',
        dest='domain_name',
        required=False,
        help='Fully qualified domain name (FQDN), such as www.example.com')
    request_cert_parser.add_argument('--validation-domain',
        dest='domain_validation_options',
        required=False,
        help='The domain name that you want ACM to use to send you emails to validate your ownership of the domain')
    request_cert_parser.add_argument('--alternative-names',
        default=[],
        dest='subject_alternative_names',
        required=False,
        nargs='+',
        help='Additional FQDNs to be included in the Subject Alternative Name extension of the ACM Certificate')
    request_cert_parser.add_argument('--generate-cli-skeleton',
        required=False,
        action='store_true',
        default=False,
        dest='generate_cli_skeleton',
        help='(boolean) Prints a sample input JSON  to  standard output')

    request_cert_parser.add_argument('--cli-input-json',
        required=False,
        action=ParseJsonInput,
        dest='cli_input_json',
        help='(boolean) Prints a sample input JSON  to  standard output')

    request_cert_parser.add_argument('--debug',
        required=False,
        action='store_true',
        default=False,
        help='(boolean) Send logging to standard output')

    confirm_cert_parser = subparsers.add_parser('confirm-certificate')
    confirm_cert_parser.set_defaults(func=_confirm_cert)
    confirm_cert_parser.add_argument('--certificate-id',
        dest='certificate_id',
        required=True,
        help='Certificate id')
    confirm_cert_parser.add_argument('--wait',
        dest='wait',
        default=5,
        type=int,
        required=False,
        help='Timeout in seconds between querying IMAP server')
    confirm_cert_parser.add_argument('--attempts',
        dest='attempts',
        type=int,
        default=1,
        required=False,
        help='Number of attempts to query IMAP server')

    confirm_cert_parser.add_argument('--debug',
        required=False,
        action='store_true',
        default=False,
        help='(boolean) Send logging to standard output')

    confirm_cert_parser.add_argument('--credentials',
        required=False,
        action=ParseIMAPCredentials,
        help='Explicitly provide IMAP credentials file')

    return parser


def main():
    is_debug = '--debug' in sys.argv[1:]

    if is_debug:
        acmagent.add_stream_log_handler(logger)
    else:
        acmagent.add_file_log_handler(logger, 'acmagent')

    parser = _setup_argparser()
    args = parser.parse_args()
    args.func(args, parser)

if __name__ == "__main__":
    main()



import json
import acmagent
import logging
import botocore
import botocore.session

logger = logging.getLogger('acmagent')


class Certificate(object):
    """
    acmagent - maps passed cli arguments to the certificate object attributes
    """
    def __init__(self, certificate_attrs):
        try:
            self.domain_name = certificate_attrs['domain_name']
            self.subject_alternative_names = certificate_attrs['subject_alternative_names']
            self.domain_validation_options = certificate_attrs['domain_validation_options']
        except KeyError as e:
            mappings = Certificate._json_mappings()
            map = {'cli': map['cli'] for name, map in mappings.items() if map['attr'] == e.args[0]}
            logger.exception('Missing certificate attribute')
            raise acmagent.MissingCertificateArgException('{} is required'.format(map['cli']))

    @classmethod
    def from_json_input(cls, cli_input_json):
        try:
            mappings = Certificate._json_mappings()
            certificate_attrs = {mappings[k]['attr']: v for k,v in cli_input_json.items()}
            return cls(certificate_attrs)
        except KeyError as e:
            logger.exception('Unknown certificate property')
            raise acmagent.InvalidCertificateJsonFileException('Unknown property {} in the specified json file'.format(e.args[0]))

    @property
    def domain_name(self):
        return self._domain_name

    @domain_name.setter
    def domain_name(self, domain_name):
        self._domain_name = domain_name

    @property
    def subject_alternative_names(self):
        return self._subject_alternative_names

    @subject_alternative_names.setter
    def subject_alternative_names(self, subject_alternative_names):
        self._subject_alternative_names = subject_alternative_names

    @property
    def domain_validation_options(self):
        return self._domain_validation_options

    @domain_validation_options.setter
    def domain_validation_options(self, domain_validation_options):
        acm_domain_validation_options = None

        if domain_validation_options:
            acm_domain_validation_options = [
                {'DomainName': alternative_name, 'ValidationDomain': domain_validation_options}
                for alternative_name in self.subject_alternative_names
            ]
            acm_domain_validation_options.append({
                    'DomainName': self.domain_name,
                    'ValidationDomain': domain_validation_options
            })

        self._domain_validation_options = acm_domain_validation_options

    @staticmethod
    def _json_mappings():
        return {
            'DomainName': {
                'attr': 'domain_name',
                'default': '',
                'cli': '--domain-name'
            },
            'ValidationDomain': {
                'attr': 'domain_validation_options',
                'default': '',
                'cli': '--validation-domain'
            },
            'SubjectAlternativeNames': {
                'attr': 'subject_alternative_names',
                'default': [],
                'cli': '--alternative-names'
            }
        }

    @staticmethod
    def template():
        template = {k: v['default'] for k,v in Certificate._json_mappings().items()}
        return json.dumps(template, sort_keys=True, indent=4, separators=(',', ': '))

    def __iter__(self):
        acm_certificate = {
            'DomainName': self.domain_name
        }

        if self.domain_validation_options:
            acm_certificate['DomainValidationOptions'] = self.domain_validation_options

        if self.subject_alternative_names:
            acm_certificate['SubjectAlternativeNames'] = self.subject_alternative_names

        for k,v in acm_certificate.items():
            yield (k, v)


class RequestCertificate(object):
    """
    Sends actual request to AWS
    """
    def __init__(self):
        self._acm_client = self._setup_acm_client()

    def request_certificate(self, certificate):
        return self._acm_client.request_certificate(**certificate)

    def _setup_acm_client(self):
        session = botocore.session.get_session()
        return session.create_client('acm')
